#!/usr/bin/perl -- -*- cperl -*-

# makeall.pl guaranteeably will sound correct, because the period is
# constant on every repetition. tones will correspondingly be purer. But
# its selection of pitches is more constrained.

use POSIX 'floor';

$cpuspeed = 39_375_000 / 22; # NTSC NES

@dmcrates = (428, 380, 340, 320, 286, 254, 226, 214, 190, 160, 142, 128,
						 106, 84, 72, 54);

@notenames = ("A-","A#","B-","C-","C#","D-","D#","E-","F-","F#","G-","G#");

$tolerance = $ARGV[0] || 7;

print "<html><body><table><tbody><tr><td>div</td><td>size</td></tr>\n";

for $divisor (2..127) {

	next if ($divisor % 16 == 0);
	# divisors with 16 as a factor are forbidden -- they can never repeat
	# evenly.

	my $stuffer;
	my $bit;
	my $comparison;
	for ($bit=0; (($bit-8) % 128) != 0 || ($bit % $divisor) != 0; $bit++) {
		if ($bit / $divisor % 2 == 1) {
			$comparison = $divisor/2 - .6; #guarantee no bias
		} else {
			$comparison = $divisor/2;
		}
		vec($stuffer,$bit,1) = $bit % $divisor < $comparison;
	}

	my $numuseful = 0;
	printf '<tr><td>%3d</td><td>%4d</td><td>-</td>',$divisor,length($stuffer);
	for $dmcrate (@dmcrates) {
		$hertz = $cpuspeed/$dmcrate/$divisor;
		$semitones = floor(.5+12*log($hertz/440)/log(2));
		$octave = 4+floor(($semitones+9)/12);
		$cents = floor(.5+1200*log($hertz/440)/log(2)) % 100;
		if ($cents > 50) {
			$cents -= 100;
		}

		if (abs($cents) < $tolerance) {
			printf '<td>%2s%1d</td>',$notenames[$semitones%12],$octave;
			$numuseful++;
		} else {
			printf '<td bgcolor="#888">%+3d</td>', $cents;
		}
	}
	print "<td>$numuseful</td></tr>\n";
	if ($numuseful > 0) {
		open OUT, ">",sprintf("trid%03d.dmc",$divisor);
		print OUT $stuffer;
		close OUT;
	}
}
